﻿using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;
using Telerik.Windows.Controls.Map;

namespace DraggablePinPoints
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
        // "is dragging started" flag
        private bool isDraggingStarted;

        // the shift from location point of PinPoint to clicked point
        private Point shiftPoint;

        // current dragged element
        private FrameworkElement draggedElement;

        // start mouse location
        private Location startLocation;

        private ObservableCollection<Employee> employees;

		public MainWindow()
		{
			InitializeComponent();

            this.InitEmployees();
        }

        private void InitEmployees()
        {
            this.employees = new ObservableCollection<Employee>();

			Employee employee = new Employee()
            {
                Name = "Name 1",
                Address = "Address 1",
                Location = new Location(39.99, -100)
            };
            this.employees.Add(employee);

            employee = new Employee()
            {
                Name = "Name 2",
                Address = "Address 2",
                Location = new Location(39.99, -100.2)
            };
            this.employees.Add(employee);

            employee = new Employee()
            {
                Name = "Name 3",
                Address = "Address 3",
                Location = new Location(40.12345, -100.3)
            };
            this.employees.Add(employee);

            this.pointsLayer.ItemsSource = this.employees;
        }

        private void OnMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            FrameworkElement element = sender as FrameworkElement;
            if (element != null)
            {
                Employee employee = element.DataContext as Employee;
                if (employee != null)
                {
                    element.CaptureMouse();
                    this.draggedElement = element;

                    Point point = e.GetPosition(this.RadMap1);

                    // get the shift from location point of element to clicked point
                    Location location = employee.Location;
                    this.startLocation = Location.GetCoordinates(this.RadMap1, point);

                    this.shiftPoint = location.GetPoint(this.RadMap1);
                    this.shiftPoint.X -= point.X;
                    this.shiftPoint.Y -= point.Y;

                    // start dragging
                    this.isDraggingStarted = true;

                    // prevent propagating event to the map
                    e.Handled = true;
                }
            }
        }

        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            if (this.isDraggingStarted)
            {
                FrameworkElement element = this.draggedElement as FrameworkElement;
                if (element != null)
                {
                    // get the current point
                    Point point = e.GetPosition(this.RadMap1);
                    point.X += this.shiftPoint.X;
                    point.Y += this.shiftPoint.Y;

                    // get the location of current point
                    Location newLocation = Location.GetCoordinates(this.RadMap1, point);

                    Employee employee = element.DataContext as Employee;
                    if (employee != null)
                    {
                        // set new location to theelement
                        employee.Location = newLocation;
                    }
                }
            }
        }

        private void OnMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (this.isDraggingStarted && this.draggedElement != null)
            {
                this.draggedElement.ReleaseMouseCapture();

                this.isDraggingStarted = false;
                this.draggedElement = null;
            }
        }
	}
}
